<?php
/*
Plugin Name: MudSlideShow
Plugin URI: http://www.sebaxtian.com/acerca-de/mudslideshow
Description: An image gallery system using Picasa and/or Flickr.
Version: 0.10.1
Author: Sebaxtian
Author URI: http://www.sebaxtian.com
*/

/*  Copyright 2007-2010  Sebaxtian  (email : sebaxtian@gawab.com)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

define('RANDOM_PHOTO', -1);
define('LAST_PHOTO', -2);
define('NO_USER', false);
define('MUDS_MNMX_V', 0.3);
define('MUDS_DB_VERSION', 1.0);

define('MUDS_LOCAL', 1);
define('MUDS_PICASA', 2);
define('MUDS_FLICKR', 3);
define('MUDS_HIGH', 100000000);
define('MUDS_XML_V', 2.1);

define('MUDS_TYPE_NONE', 0);
define('MUDS_TYPE_PICASA', 1);
define('MUDS_TYPE_FLICKR', 2);

define('MUDS_FILE_LIST', 1);
define('MUDS_FILE_ALBUM', 2);
define('MUDS_FILE_TAG', 3);
define('MUDS_FILE_FEED', 4);

define('MUDS_WIDGET_RANDOM',1);
define('MUDS_WIDGET_LAST',2);
define('MUDS_WIDGET_RANDOMLAST',3);
define('MUDS_WIDGET_RANDOMFROM',4);

//Set the viewer functions
$options = get_option('muds_options');
switch($options['gallery_type']) {
	case '1':
		require('viewers/highslide-viewer.php');
		break;
	case '2':
		require('viewers/lytebox-viewer.php');
		break;
	default:
		require('viewers/none-viewer.php');
		break;
}

add_action('init', 'muds_add_buttons');
add_action('init', 'muds_text_domain',1);
add_action('admin_menu', 'muds_menus');
add_action('wp_head', 'muds_header');
add_action('activate_plugin', 'muds_activate');
add_action('wp_footer', 'muds_footer');
add_filter('the_content', 'muds_content');

//The activation error routine
if ($_GET['action'] == 'error_scrape') {
	//Activate the i18n
	muds_text_domain();
	
	//Error 1: you don't have minimax installed
	if(!function_exists('minimax') || minimax_version()<0.3) {
		die(sprintf( printf(__('You have to install <a href="%s" target="_BLANK">minimax %1.1f</a> in order for this plugin to work.', 'mudslide'), "http://wordpress.org/extend/plugins/minimax/", MUDS_MNMX_V) ));
	}
	
	//Get the cache directory path
	$ans = parse_url(get_bloginfo('wpurl'));
	$cache_dir = $_SERVER['DOCUMENT_ROOT'].$ans['path'].'/wp-content/' . MNMX_CACHE;
	
	//Error 2: we can't create the cache directory
	if(!file_exists($cache_dir.'/mudslideshow')) {
		die(sprintf(__("Can't create the cache dir: %s<br>You have to create it manually.", 'mudslide'), $cache_dir.'/lexi/'));
	}
	
	//Error 3: we can't write in the cache directory
	if($handle = @fopen($cache_dir.'/mudslideshow/test.txt','w')) {
		fclose($handle);
		unlink($cache_dir.'/mudslideshow/test.txt');
	} else {
		die(sprintf(__("Can't write in the cache dir: %s<br>You have to modify the permissions manually.", 'mudslide'), $cache_dir.'/lexi/'));
	}
}

include_once(ABSPATH . WPINC . '/rss.php');
@include_once(ABSPATH . WPINC . '/class-simplepie.php');

require_once('sources/picasa.php');
require_once('sources/flickr.php');

/**
* To declare where are the mo files (i18n).
* This function should be called by an action.
*
* @access public
*/
function muds_text_domain() {
	load_plugin_textdomain('mudslide', 'wp-content/plugins/mudslideshow/lang');
}

/**
* Function to return the url of the plugin concatenated to a string. The idea is to
* use this function to get the entire URL for some file inside the plugin.
*
* @access public
* @param string str The string to concatenate
* @return The URL of the plugin concatenated with the string 
*/
function muds_plugin_url($str = '')
{

	$aux = '/wp-content/plugins/mudslideshow/'.$str;
	$aux = str_replace('//', '/', $aux);
	$url = get_bloginfo('wpurl');
	return $url.$aux;

}

/**
* Function to return the path to the cache concatenated to a string. The idea is to
* use this function to get the entire path for some cache file.
*
* @access public
* @param string str The string to concatenate
* @return The path of the plugin concatenated with the string 
*/
function muds_plugin_cache($str = '')
{
	$ans = parse_url(get_bloginfo('wpurl'));
	$aux = $_SERVER['DOCUMENT_ROOT'].$ans['path'].'/wp-content/'.MNMX_CACHE.'/mudslideshow/'.$str;
	$aux = str_replace('//', '/', $aux);
	return $aux;

}

/**
* Function to create the database and to add options into WordPress
* This function should be called by an action.
*
* @access public
*/
function muds_activate()
{
	global $wpdb;
	
	//Error 1: you don't have minimax installed
	if(!function_exists('minimax') || minimax_version()<0.3) {
		trigger_error('Muds-error: #1', E_USER_ERROR);
	}
	
	//Create the cache directory if it doesn't exist
	$ans = parse_url(get_bloginfo('wpurl'));
	$cache_dir = $_SERVER['DOCUMENT_ROOT'].$ans['path'].'/wp-content/'.MNMX_CACHE;
	
	if(!file_exists($cache_dir)) mkdir($cache_dir);
	if(!file_exists($cache_dir.'/mudslideshow')) mkdir($cache_dir.'/mudslideshow');
	
	//Error 2: we can't find or create the cache directory.
	if(!file_exists($cache_dir.'/mudslideshow')) {
		trigger_error('Muds-error: #2', E_USER_ERROR);
	}
	
	//Error 3: we can't write in the cache directory
	if($handle = fopen($cache_dir.'/mudslideshow/test.txt','w')) {
		fclose($handle);
		unlink($cache_dir.'/mudslideshow/test.txt');
	} else {
		trigger_error('Muds-error: #3', E_USER_ERROR);
	}
	
	$options = array( 
		'gallery_type' => 0,
		'columns' => 5,
		'rows' => 4,
		'user_picasa' => '',
		'user_flickr' => '',
		'show_link' => false,
		'extend' => false,
		'last_source' => MUDS_TYPE_PICASA
	);
	
	add_option('muds_options', $options);
	
	$table_name = $wpdb->prefix . "mudslide";
	if($wpdb->get_var("show tables like '$table_name'") != $table_name)
	{

		$sql = "CREATE TABLE $table_name(
			id bigint(1) NOT NULL AUTO_INCREMENT,
			type int NOT NULL,
			file text NOT NULL,
			name text NOT NULL,
			PRIMARY KEY (id)
			);";

		require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
		dbDelta($sql);
		add_option('muds_db_version', MUDS_DB_VERSION);
	}
	
	//Update to multiwidget	
	muds_multiWidget();
	
	//Update to new configuration array.
	muds_newConfig();
	
}


/**
* Update to new configuration array.
*
* @access public
*/
function muds_newConfig() {
	//Do we have the old configuration rows?
	if( get_option('muds_user') ) {
		$options = get_option('muds_options');
		$options['gallery_type'] = get_option('muds_gallery_type');
		delete_option('muds_gallery_type');
		$options['columns'] = get_option('muds_columns');
		delete_option('muds_columns');
		$options['rows'] = get_option('muds_rows');
		delete_option('muds_rows');
		if($options['rows']==0) $options['rows'] = 4;
		$options['user_picasa'] = get_option('muds_user');
		delete_option('muds_user');
		$options['show_link'] = get_option('muds_showlink');
		delete_option('muds_showlink');
		$options['extend'] = get_option('muds_extend');
		delete_option('muds_extend');
		
		update_option('muds_options', $options);
	}
}

/**
* Update to multiwidget.
*
* @access public
*/
function muds_multiWidget() {
	//If we have the widget from 0.9.3, use it with the new name
	$settings = get_option('widget_muds_widget');
	if ( is_array($settings) ) {
		//Change the gallery to gallery_id
		foreach ( (array) $settings as $index => $widget ) {
			if ( is_array($widget) && count($widget)>1) {
				$settings[$index]['gallery_id']=$settings[$index]['gallery'];
				unset($settings[$index]['gallery']);
			}
		}
		
		update_option('widget_mudslide', $settings);
		
		//Update the sidebar to use the new widget name
		$sidebars_widgets = get_option('sidebars_widgets');
		foreach ( (array) $sidebars_widgets as $index => $sidebar ) {
			if ( is_array($sidebar) ) {
				foreach ( $sidebar as $i => $name ) {
					//Explode the name
					$name_aux=explode('-', $name); 
					//Check if the widget has the name from the old one
					if ( $name_aux[0] == 'muds_widget' && count($name_aux)>1) {
						//Use the new name with the old number
						$sidebars_widgets[$index][$i] = "mudslide-".$name_aux[1];
						$changed = true;
					}
				}
			}
			if($changed) update_option('sidebars_widgets', $sidebars_widgets);
		}
		
		//We have updated it,so  we don't need it anymore
		delete_option('widget_muds_widget');
	} else { //No, we don't have the widget from version 0.9.3
		//Update the sidebar for multiwidget
		$base_name='mudslide';
		$settings = get_option('widget_'.$base_name);
		if ( !array_key_exists('_multiwidget', $settings) ) { //If we don't have a multiwidget, create it
			$changed = false;
			//Update the sidebar to use the first item in the multiwidget array
			$sidebars_widgets = get_option('sidebars_widgets');
			foreach ( (array) $sidebars_widgets as $index => $sidebar ) {
				if ( is_array($sidebar) ) {
					foreach ( $sidebar as $i => $name ) {
						//Check if the widget has the name from the single one
						if ( $name == $base_name.'-widget' ) {
							//Use the first item in the multiwidget array, it means item number 2
							$sidebars_widgets[$index][$i] = "$base_name-2";
							$changed = true;
						}
					}
				}
			}
			if($changed) update_option('sidebars_widgets', $sidebars_widgets);
		}
	}
}

/**
* Enable menus.
* This function should be called by an action.
*
* @access public
*/
function muds_menus() {
	add_management_page('MudSlideShow', 'MudSlideShow', 10, 'mudsmanage', 'muds_manage_page');
	add_options_page('MudSlideShow', 'MudSlideShow', 10, 'mudsoptions', 'muds_options');
}


/**
* Return the file of a gallery in the internal list.
*
* @access public
* @param int id ID of the gallery in the internal list.
* @return string The filename of the gallery. False if it doesn't exists.
*/
function muds_get_gallery_file($id) {
	global $wpdb;
	$answer=false;
	$table_name = $wpdb->prefix . "mudslide";
	if($aux = $wpdb->get_var("SELECT file FROM $table_name WHERE id=$id")) {
		$answer=$aux;
	}
	return $answer;
}

/**
* Return the type of a gallery in the internal list.
*
* @access public
* @param int id ID of the gallery in the internal list.
* @return string The gallery type. False if it doesn't exists.
*/
function muds_get_gallery_type($id) {
	global $wpdb;
	$answer=false;
	$table_name = $wpdb->prefix . "mudslide";
	if($aux = $wpdb->get_var("SELECT type FROM $table_name WHERE id=$id")) {
		$answer=$aux;
	}
	return $answer;
}

/**
* Return the name of a gallery in the internal list.
*
* @access public
* @param int id ID of the gallery in the internal list.
* @return string The gallery name. False if it doesn't exists.
*/
function muds_get_gallery_name($id) {
	global $wpdb;
	$answer=false;
	$table_name = $wpdb->prefix . "mudslide";
	if($aux = $wpdb->get_var("SELECT name FROM $table_name WHERE id=$id")) {
		$answer=$aux;
	}
	return $answer;
}

/**
* Deletes a gallery from the internal list.
*
* @access public
* @param int id ID of the gallery in the internal list.
* @return bool True on succes. False if not.
*/
function muds_delete_gallery($id) {
	global $wpdb;
	$answer=false;
	$table_name = $wpdb->prefix . "mudslide";
	if($aux = $wpdb->get_results("SELECT type, file FROM $table_name WHERE id=$id")) {
		$type=$aux[0]->type;
		$filename=$aux[0]->file;
		$del = @unlink(muds_plugin_cache($filename));
		$db = $wpdb->query("DELETE FROM $table_name WHERE id=$id");
		if($del && $db) $answer = true;
	}
	return $answer; 
}

/**
* Updates a gallery in the internal list.
*
* @access public
* @param int id ID of the gallery in the internal list.
* @return bool True on succes. False if not.
*/
function muds_update_gallery($id) {
	global $wpdb;
	$answer=false;
	
	//Get the data from this gallery
	$table_name = $wpdb->prefix . "mudslide";
	if($aux = $wpdb->get_results("SELECT type, file FROM $table_name WHERE id=$id")) {
		$type=$aux[0]->type;
		$filename=$aux[0]->file;
		
		switch($type) {
			case MUDS_PICASA:	//If this is a picasa album
				$answer = mudsPicasa_update_gallery_by_filename($filename);
				break;
			case MUDS_FLICKR:	//If this is a flickr album
				$answer = mudsFlickr_update_gallery_by_filename($filename);
				break;
		}
	}
	return $answer;
}


/**
* XML Entity Mandatory Escape Characters
*
* @access public
* @param string string The string to change
* @return string The chabged string
*/
if (! function_exists ('mb_htmlentities'))
{
   function mb_htmlentities ($string, $quote_style = ENT_COMPAT, $charset = 'UTF-8', $double_encode = true)
   {
		$version = explode('.', PHP_VERSION);
		$version = $version[0] * 10000 + $version[1] * 100 + $version[2];
		if($version < 50203)
   		return htmlentities ($string, $quote_style, $charset);
     	else
     		return htmlentities ($string, $quote_style, $charset, $double_encode);
   }
}

/**
* XML Entity Mandatory Escape Characters
*
* @access public
* @param string string The string to change
* @return string The chabged string
*/
function xmlentities($string) { 
	$string = (string) $string;
   return str_replace ( array ( '&', '"', "'", '<', '>' ), array ( '&amp;' , '&quot;', '&#39;' , '&lt;' , '&gt;' ), $string ); 
} 

/**
* Page to manage the galleries likst.
*
* @access public
*/
function muds_manage_page()
{
	global $wpdb;
	$table_name = $wpdb->prefix . "mudslide";
	$messages=array();
	if(!function_exists('minimax_version') || minimax_version()<MUDS_MNMX_V) { //Do we have minimax?
		array_push($messages, sprintf(__('You have to install <a href="%s" target="_BLANK">minimax %1.1f</a> in order for this plugin to work.', 'mudslide'), "http://wordpress.org/extend/plugins/minimax/", MUDS_MNMX_V ));
	}

	$mode_x=$_POST['mode_x']; //Something to execute?
	$mode=$_GET['mode']; //Something to show?
	
	$doaction=false; //CLicked the button for bulk action?
	if($_POST['doaction']!="") $doaction=$_POST['action']; //Top button
	if($_POST['doaction2']!="") $doaction=$_POST['action2']; //Bottom button
	if($doaction) {
		switch($doaction) {
			case 'delete': //Delete som galleries
				if(count($_POST['checked_galleries'])>0) {
					foreach($_POST['checked_galleries'] as $checked_id) { //Delete each gallery
						muds_delete_gallery($checked_id);
					}
				}
				break;
		}
	}
	
	switch($mode) { //Something to do?
		case 'delete': //Delete one gallery
			check_admin_referer('mudslide_deletegallery');
			$id=$_GET['id'];
			//If deleted, show the message too.
			if(muds_delete_gallery($id)) array_push($messages, __("Gallery deleted",'mudslide'));
			break;
	}
	
	//Do we have to show a message?
	if(count($messages)>0) {
		echo "<div class='updated'>";
		foreach($messages as $message) echo "<p><strong>$message</strong></p>";
		echo "</div>";
	}

	// Now display the options editing screen
	// options form
	include('templates/mudslide_manage.php');
}

/**
* Page to manage the options.
*
* @access public
*/
function muds_options()
{
	global $wpdb;
	$messages=array();

	$options = get_option('muds_options');

	$mode_x=$_POST['mode_x']; //Someting to execute?
	$mode=$_GET['mode']; //Something to show?
	
	switch($mode_x) {
		case 'manage_x': //Update the config data
			$mode='manage';
			
			$options['gallery_type'] = $_POST['muds_gallery_type'];
			$options['columns'] = $_POST['muds_columns'];
			$options['rows'] = $_POST['muds_rows'];
			$options['show_link'] = $_POST['muds_showlink'];
			$options['extend'] = $_POST['muds_extend'];
						
			// Save the posted value in the database
			update_option( 'muds_options', $options);

			// Put an 'options updated' message on the screen
			array_push($messages,__( 'Options saved', 'mudslide' ));
			
			break;
	}
	
	switch($mode) {
		case 'manage':
			break;
	}
	
	//Do we have messages to show?
	if(count($messages)>0) {
		echo "<div class='updated'>";
		foreach($messages as $message) echo "<p><strong>$message</strong></p>";
		echo "</div>";
	}
	
	//Ok, show the dialog
	include('templates/mudslide_options.php');
}

/**
* Filter to manage contents. Check for [mudslide] tags.
* This function should be called by a filter.
*
* @access public
* @param string content The content to change.
* @return string The content with the changes the plugin have to do.
*/
function muds_content($content) {
	//global $wpdb;
	$options = get_option('muds_options');
	
	//The images!
	if($options['extend']) {
		$search = "/(?:<p>)*\s*\[caption(.*?)align=('|\")align([^>]*)('|\")(.*?)width=('|\")([\d]*)('|\")(.*?)caption=('|\")([^>]*)('|\")\]<a(.*?)href=('|\")([^>]*).(bmp|gif|jpeg|jpg|png)('|\")(.*?)><img(.*?)title=('|\")([^>]*)('|\")(.*?)src=('|\")([^>]*).(bmp|gif|jpeg|jpg|png)('|\")(.*?)><\/a>\[\/caption\]\s*(?:<\/p>)*/i";
		if(preg_match_all($search, $content, $matches)) { //search for mudslide picasa
			if (is_array($matches)) { //We found
				foreach ($matches[1] as $key =>$v0) { //for each gallery, do
					//Get the values
					$search = $matches[0][$key];
					$float = $matches[3][$key];
					$size = $matches[7][$key];
					$caption = $matches[11][$key];
					$image  = $matches[15][$key].'.'.$matches[16][$key];
					$title  = $matches[21][$key];
					$thumbnail = $matches[25][$key].'.'.$matches[26][$key];
					
					if(strlen($caption)>0) $title = $caption;
					
					$replace = "<p>".muds_show($image, $thumbnail, $title, mt_rand(111111,999999), $float, false, $size, false, MUDS_TYPE_NONE)."</p>";
					$content = str_replace ($search, $replace, $content);
				}
			}
		}
	}
	
	/*$replace = mudsPicasa_show_photo( $user, $gallery, $photo, $size, $float, $conf ); //Show the photo
   $replacement = '<a$1href=$2$3.$4$5 class="highslide-image" onclick="return hs.expand(this);"$6>$7</a>';
   $content = preg_replace($search, $replacement, $content);*/

	//MudSlideShow - Gallery
	$search = "/(?:<p>)*\s*\[mudslide:(picasa|flickr),(\d+),(htt[^,]+|([\w.@]+),([\w.@]+|\d+|[\w#]+))(,\d+,\d+|,(\d+c|\d+|s|t|m|l),(left|right|center)|)?\]\s*(?:<\/p>)*/i";
	if(preg_match_all($search, $content, $matches)) { //search for mudslide picasa
		if (is_array($matches)) { //We found
			foreach ($matches[1] as $key =>$v0) { //for each gallery, do
				
				//Get the values
				$search = $matches[0][$key];
				
				$type=$matches[1][$key];
				$conf=$matches[2][$key];
				
				$user = $matches[3][$key];
				$gallery=1;
				if($matches[5][$key]) {
					$user = $matches[4][$key];
					$gallery = $matches[5][$key];
				}
				
				$last=$matches[6][$key];
			
				$begin=0;
				$end=0;
				$simple=false;
				
				//If we have data at the end, search if it is for the begin and the end or
				//for a simple gallery 
				if($last) {
					$data=explode(',', $last);
					if(is_numeric($data[2])) { //This is begin-end data
						$begin=$data[1];
						$end=$data[2];
						if($first>$last) {
							$first=$last=0;
						}
					} else { //Simple gallery
						$simple=true;
						$size=$data[1];
						$float=$data[2];
					}
					
				}
				
				if(function_exists('minimax_version') && minimax_version()>=MUDS_MNMX_V) { //Do we have minimax?
					if(!$simple) { //If it is not a simple gallery, show the gallery, else show simple
						switch($type) {
							case 'picasa': 
								$replace = mudsPicasa_show_gallery( $user, $gallery, $begin, $end, $conf );
								break;
							case 'flickr': 
								$replace = mudsFlickr_show_gallery( $user, $gallery, $begin, $end, $conf );
								break;
						}
					} else {
						switch($type) {
							case 'picasa':
								$replace = mudsPicasa_show_simple($user, $gallery, $size, $float, $conf);
								break;
							case 'flickr':
								$replace = mudsFlickr_show_simple($user, $gallery, $size, $float, $conf);
								break;
						}
					}
				} else { //If we don't have minimax, ask for it to the user.
					$replace = sprintf(__('You have to install <a href="%s" target="_BLANK">minimax %1.1f</a> in order for this plugin to work.', 'mudslide'), "http://wordpress.org/extend/plugins/minimax/", MUDS_MNMX_V);
				}
				//Ok, done with this, go ahead
				$replace = "<p>$replace</p>"; 
				$content= str_replace ($search, $replace, $content);
			}
		}
	}

	//MudSlideShow - Photo
	$search = "/(?:<p>)*\s*\[mudslide:(picasa|flickr),(\d+),(htt[^,]+|([\w.@]+),([\w.@]+|\d+|[\w#]+)),(\d+),(\d+|\d+c|s|t|m|l),(left|right|center)?\]\s*(?:<\/p>)*/i";
	if(preg_match_all($search, $content, $matches)) { //search for picasa photo
		if (is_array($matches)) { //we found
			foreach ($matches[1] as $key =>$v0) { //foreach photo, do
				//get the values
				$search = $matches[0][$key];
				$type=$matches[1][$key];
				$conf=$matches[2][$key];
				
				$user = $matches[3][$key];
				$gallery=1;
				if($matches[5][$key]) {
					$user = $matches[4][$key];
					$gallery = $matches[5][$key];
				}
				
				$photo=$matches[6][$key];
				$size=$matches[7][$key];
				$float=$matches[8][$key];

				if(function_exists('minimax_version') && minimax_version()>=MUDS_MNMX_V) { //Do we have minimax?
					switch($type) {
						case 'picasa':
							$replace = "<p>".mudsPicasa_show_photo( $user, $gallery, $photo, $size, $float, $conf )."</p>"; //Show the photo
							break;
						case 'flickr':
							$replace = "<p>".mudsFlickr_show_photo( $user, $gallery, $photo, $size, $float, $conf )."</p>"; //Show the photo
							break;
					}
				} else {
					//If we don't have minimax, ask for it to the user.
					$replace = sprintf(__('You have to install <a href="%s" target="_BLANK">minimax %1.1f</a> in order for this plugin to work.', 'mudslide'), "http://wordpress.org/extend/plugins/minimax/", MUDS_MNMX_V);
				}
				$content = str_replace ($search, $replace, $content);
			}
		}
	}

	return $content;
}

/**
* Returns an array with filename, gallery_id and md5
* @acces private
* @param string user The user id or feed url
* @param string gallery The gallery id
* @return array with filename, gallery_id and md5
**/
function muds_detect_type($user, $gallery, $source) {
	$md5 = false;
	
	$search = "*******";
	switch($source) {
		case MUDS_TYPE_PICASA:
			$search = "http://picasaweb.google.com";
			$letter = "p";
			break;
		case MUDS_TYPE_FLICKR:
			$search = "http://api.flickr.com";
			$letter = "f";
			break;
	}
	
	$filename = $letter."t-$gallery.xml"; //We suppose it's a tag
	$album_type = MUDS_FILE_TAG; 
	
	if(strstr($user, $search)) { //An RSS feed
		//$user = str_replace('&amp;', '&', $user);
		$md5 = md5($user);
		$filename = $letter."f-$md5.xml";
		$gallery = 1;
		$album_type = MUDS_FILE_FEED;
	}
	
	if(strcmp($gallery, "0")==0 || strcmp($gallery,$user)==0) { //Zero gallery
		$gallery=$user;
		$filename = $letter."a-$gallery.xml";
		$album_type = MUDS_FILE_ZERO;
	}
	
	if(is_numeric($gallery) && !$md5) { //If the gallery is numeric and we don't have an md5, it means is an album
		$filename = $letter."a-$gallery.xml";
		$album_type = MUDS_FILE_ALBUM;
	}
	
	return compact("filename", "gallery", "md5", "album_type");
}

/**
* Create the HTML code for a simple gallery. If first and last are equal to 0, show all the gallery
*
* @access public
* @param xml data The data description of the gallery.
* @param string update_post The post elements for the data.php script to update this gallery.
* @param int first The first foto to show
* @param int first The last foto to show
* @param int conf A configuration number for future releases.
* @return string The HTML code for the simple frame viewer.
*/
function muds_gallery($data, $update_post, $first=0, $last=0, $conf, $type) {
	global $wpdb, $post, $current_user;
	
	//The configuration data to show the gallery
	$options = get_option('muds_options');
	$muds_type = $options['gallery_type'];
	$muds_columns = $options['columns'];
	$muds_showlink = $options['show_link']; 
	
	$rand = mt_rand(111111,999999); //The identifier
	$answer="";
	$count=1; //Count the number of the picture
	$column=1; //Counter to determine if we have to jump
	$table="";
	
	//Select icon
	switch($type) {
		case MUDS_TYPE_PICASA:
			$icon = muds_plugin_url('img/source-picasa.gif');
			$icon = "Picasa"; //"<img src='$icon' alt='Picasa' border='0'>";
			$thumb_size = "72-c";
			$full_size = 800;
			break;
		case MUDS_TYPE_FLICKR:
			$icon = muds_plugin_url('img/source-flickr.gif');
			$icon = "Flickr"; //"<img src='$icon' alt='Picasa' border='0'>";
			$thumb_size = 's';
			$full_size = '_';
			break;
	}
	
	//The nonce value
	$nonce = wp_create_nonce('mudslide');
	
	if($photo_gallery = new SimpleXMLElement($data)) { //Parse the data
		$answer="";
		$attr = $photo_gallery->attributes();
		$gallery_url = $attr->url;
		
		//Get the link to external gallery
		$link = "<a href='$gallery_url' target='_BLANK'>".sprintf(__('Open in %s', 'mudslide'), $icon)."</a>";
		
		if($first==0 || $last==0) { //If we don't have begin and end, show all the gallery
			$first=1;
			$last=count($photo_gallery->photo);
		}
		
		foreach($photo_gallery->photo as $photo) { //Go trhough the gallery
			if($count>=$first && $count<=$last) { //If the photo is between the first and the last picture to show
				if($column==1) { //Are we starting a row?
					$table.="<tr>"; //We start the row
				}
				
				//Get the data
				$media = str_replace('%size%', $full_size, $photo->resize);
				$thumbnail = str_replace('%size%', $thumb_size, $photo->resize);
				if($type == MUDS_TYPE_FLICKR) $media = str_replace('__', '', $media);
				$attr2 = $photo->attributes();
				$url = $attr2->url;
				$title=$photo->title;
				$caption = false;
				if($conf & 1) { //Should we show the comment
					$caption = $photo->comment;
				}
				$table.="<td align='center'><span class='slide'>".muds_show($media, $thumbnail, $title, $rand, 'center', $caption, 72, $url, $type)."</span></td>"; //Create a cell with the photo
				if($column==$muds_columns) { //is this the last picture from the row
					$table.="</tr>"; //close the row
					$column=0; //Start the counter again
				}

				$column++; //Add 1 to the column counter
			}
			$count++; //Add 1 to the picture counter
		}
		
		//No more pictures
		if($column!=1) { //Did we left a row opened?
			$table.="</tr>"; //Close the opened row
		}
		
		$sourcelink = "";
		if($muds_showlink) $sourcelink = "<div class='throbber-off'>$link</div>";
		
		//Section to update the gallery
		if($post->post_author == $current_user->id || current_user_can('edit_others_posts')) { //Can this user edit the post
			
			$updatelink = "<div id='throbber-msspage$rand' class='throbber-off'><a style='cursor : pointer;' id='update-$rand' onclick=\"
					var update_gallery$rand = function() {
						if (mss$rand.xhr.readyState == 4 && mss$rand.xhr.status==200 ) {
							window.location.reload();
						}
					}
					document.getElementById('update-$rand').innerHTML='".__('Updating', 'mudslide')."';
					var mss$rand=new minimax('".muds_plugin_url('/ajax/update.php')."', false);
					mss$rand.setThrobber('throbber-msspage$rand', 'throbber-on', 'throbber-off');
					mss$rand.setFunc(update_gallery$rand);
					mss$rand.post('nonce=$nonce&amp;$update_post');\">".__('Update','mudslide')."</a></div>";
			
			//Add a minimax function to update the gallery
			$table.="<tr><td colspan='$muds_columns' align='right'>$sourcelink $updatelink</td></tr>";
		} else {
			if($muds_showlink) $table.= "<tr><td colspan='$muds_columns' align='right'><small>$sourcelink</small></td></tr>";
		}
		
		//We have owr table!!!!!
		$answer = "<div><table class='mudslideshow'>$table</table></div>";	
	}
	
	return $answer;
}

/**
* Create the HTML code for a simple gallery
*
* @access public
* @param xml data The data description of the gallery.
* @param string update_post The post elements for the data.php script to update this gallery.
* @param int size The size of the frame.
* @param string float The side where the frame has to float.
* @param int conf A configuration number for future releases.
* @return string The HTML code for the simple frame viewer.
*/
function muds_simple($data, $update_post, $size, $float, $conf, $type) {

	global $post, $current_user;

	switch($type) {
		case MUDS_TYPE_PICASA:
			$size_aux = str_replace('-c', '', $size);
			break;
		case MUDS_TYPE_FLICKR:
			switch($size) {
				case 's':
					$size_aux=75;
					break;
				case 't':
					$size_aux=100;
					break;
				case 'm':
					$size_aux=240;
					break;
				case 'l':
					$size_aux=500;
					break;	
			}
			break;
		
	}
	
	//The nonce value
	$nonce = wp_create_nonce('mudslide');
	
	//Calculate the position for the buttons
	$box_height=$size_aux+10;
	$box_width=$size_aux+10;
	$slidemeter_left=$size_aux-110;
	$slidemeter_top=$size_aux-35;
	
	$id=rand(1000000,9999999);
	$rand = $id;
	$throbber="<div id='slidemeter$id' class='slidemeter'></div>";
		
	//Section to update the gallery
	if($post->post_author == $current_user->id || current_user_can('edit_others_posts')) { //can this user edit the post
		//Add a minimax function to update the gallery
		$throbber="<div id='throbber-msspage$rand' class='throbbersimp-off' onclick=\"var update_gallery$rand = function() {
			if (mss$rand.xhr.readyState == 4 && mss$rand.xhr.status==200 ) {
				window.location.reload();
			}
		}
		var mss$rand=new minimax('".muds_plugin_url('/ajax/update.php')."', false);
		mss$rand.setThrobber('throbber-msspage$rand', 'throbbersimp-on', 'throbbersimp-off');
		mss$rand.setFunc(update_gallery$rand);
		mss$rand.post('nonce=$nonce&amp;$update_post');\" />$throbber</div>
		";
	}
	
	//Data to align the frame
	if($float=='center') {
		$center=" align=\"center\"";
	}
	$class="";
	if($float) {
		$class=" class=\"$align\"";
	}	
	
	if($photo_gallery = new SimpleXMLElement($data)) { //Parse the data
		//Create the simple frame	
		$answer="<div id='simple_gallery$id' style='background : url(".muds_plugin_url('/img/simpleslide/background.png').") no-repeat center center; border-style : solid; border-width : 1px; height : ".$box_height."px; width : ".$box_width."px;'>
			<div style='left : ".$slidemeter_left."px; position : relative; top : ".$slidemeter_top."px; text-align: left;'><input type='hidden' id='actualPic$id' value='0'/><input type='hidden' id='path$id' value='".muds_plugin_url('/img/simpleslide/')."' />
			<table width='100' style='background : url(".muds_plugin_url('/img/simpleslide/background.png').");'>
			<tr>
			<td width='25' align='center'>
			<input type='image' id='backslide$id' src='".muds_plugin_url('/img/simpleslide/leftarrow.png')."' onclick='doPrevious($id,slideImages$id);' />
			</td>
			<td align='center'>$throbber</td>
			<td width='25' align='center'>
			<input type='image' id='nextslide$id' src='".muds_plugin_url('/img/simpleslide/rightarrow.png')."' onclick='doNext($id,slideImages$id);' />
			</td>
			</tr>
			</table>
			</div>
			<script type='text/javascript'>
			var slideImages$id = new Array(";
		//Create the array of photos
		$first=true;
		$first_image="";
		foreach($photo_gallery->photo as $photo) {
			//Get the photo 
			$media = str_replace('%size%', $size, $photo->resize);
			if($type==MUDS_TYPE_FLICKR && $size=='l') $media = str_replace('_%size%', '', $photo->resize);
			
			if(!$first) {
				$answer.=",";
			} else {
				$first=false;
				$first_image=$media;
			}
			$answer.="'".$media."'";
		}
		$answer.=")
			var actualPic=document.getElementById('actualPic$id').value;
			var div=document.getElementById('simple_gallery$id');
			div.style.backgroundImage='url(".$first_image.")';
			arrows($id,slideImages$id);
			changeText($id,slideImages$id);
			preloadImages(slideImages$id);
			</script>
			</div>";
		if($float=='center') $float="";
		
		//Ok, we have owr frame!!!!
		$answer="<div class=\"slide$float\"".$center.">$answer</div>";
	}
	
	return $answer;

}

/**
* Create the HTML code for a simple photo
*
* @access public
* @param xml data The data description of the gallery.
* @param int p_id The position of the photo in the gallery. Can be also LAST_PHOTO or RANDOM_PHOTO.
* @param int size The size of the photo in the post.
* @param string float Where the photo has to float.
* @param int conf A numeric value for the configuration.
* @return string The HTML code for the gallery the photos, false if not. 1 to show the first comment
	in picasaweb as a big description. 
*/
function muds_photo($data, $p_id, $size, $float, $conf, $type) {
	
	$answer = false;
	if($photo_gallery = new SimpleXMLElement($data)) { //Parse the data
		$count = count($photo_gallery->photo);
		if($count>0) {
			if($p_id==RANDOM_PHOTO) $p_id=mt_rand(0,$count); // Show a random photo?
			if($p_id==LAST_PHOTO) $p_id=$count-1; //Show the last photo?
			if($count>=$p_id) { //Do the gallery have a photo in this position?
				$photo=$photo_gallery->photo[$p_id-1]; //Ok, get the photo
				$answer="";
				$thumbnail = str_replace('%size%', $size, $photo->resize);
				
				//Get the data
				switch($type) {
					case MUDS_TYPE_PICASA:
						$media = str_replace('%size%', 800, $photo->resize);
						break;
					case MUDS_TYPE_FLICKR:
						if($size<76) $thumbnail = str_replace('%size%', 's', $photo->resize);
						if($size>75 && $size<151) $thumbnail = str_replace('%size%', 't', $photo->resize);
						if($size>150 && $size<241) $thumbnail = str_replace('%size%', 'm', $photo->resize);
						if($size>240) $thumbnail = str_replace('_%size%', '', $photo->resize);
						$media = str_replace('_%size%', '', $photo->resize);
						break;
				}
				
				$attr = $photo->attributes();
				$url = $attr->url;
				$title = $photo->title;
				$caption=false;
				if($conf & 1) { //Should we show the comment?
					$caption = $photo->comment;
				}
				$answer=muds_show($media, $thumbnail, $title, mt_rand(111111,999999), $float, $caption, $size, $url, $type);
					
			} else { //You are asking for something that is not here.
				$answer=__('Photo doesn\'t exist', 'mudslide' );
			}
		} else {
			$answer=__('This album doesn\'t have photos', 'mudslide' );
		}
	}	
	return $answer;
}

/**
* Enable buttons in tinymce.
* This function should be called by an action.
*
* @access public
*/
function muds_add_buttons() {
	// Don't bother doing this stuff if the current user lacks permissions
	if ( !current_user_can('edit_posts') && !current_user_can('edit_pages') ) return;

	// Add only in Rich Editor mode
	if ( get_user_option('rich_editing') == 'true') {
	 
		// add the button for wp21 in a new way
		add_filter('mce_external_plugins', 'add_mudslide_script');
		add_filter('mce_buttons', 'add_mudslide_button');
	}
}

/**
* Function to add the button to the bar.
* This function should be called by a filter.
*
* @access public
*/
function add_mudslide_button($buttons) {

	array_push($buttons, 'MudSlide');
	return $buttons;

}

/**
* Function to set the script which should answer when the user press the button.
* This function should be called by a filter.
*
* @access public
*/
function add_mudslide_script($plugins) {
	$pluginURL = muds_plugin_url('/tinymce/editor_plugin.js');
	$plugins['MudSlide'] = $pluginURL;
	return $plugins;
}

/**
* Return the HTML code for the photo in the widget.
*
* @access public
* @param int gallery The id of the gallery in the internal list.
* @param int photo_id The position of the photo in the gallery.
* @return string The HTML code. Empty if error. 
*/
function muds_widget_photo($gallery, $photo_id, $size) {
	$type = muds_get_gallery_type($gallery);
	$answer="";
	switch($type) {
		case MUDS_PICASA: 
			$answer = mudsPicasa_widget_photo(muds_get_gallery_file($gallery), $photo_id, $size);
			break;
		case MUDS_FLICKR: 
			$answer = mudsFlickr_widget_photo(muds_get_gallery_file($gallery), $photo_id, $size);
			break;
		default:
			$answer=__('Unsupoerted source type', 'mudslide');
			break;
	}
	return $answer;
}

/**
* Return the HTML for the widget, using the configuration variables.
* If minimax isn't installed, this function returns a warning.
*
* @access public
* @return string The HTML code. Empty if error. 
*/
function muds_widget_contents( $instance ) {
	global $wpdb;
	$table_name = $wpdb->prefix . "mudslide";
	$answer=__('Album doesn\'t exist', 'mudslide' );
	if(function_exists('minimax_version') && minimax_version()>=MUDS_MNMX_V) { //Do we have minimax?
		switch($instance['type']) {
			case MUDS_WIDGET_RANDOM:
				if($gallery_id=$wpdb->get_var("SELECT id FROM $table_name ORDER BY RAND() LIMIT 1"))
					$answer=muds_widget_photo($gallery_id,RANDOM_PHOTO, $instance['size']);
				break;
			case MUDS_WIDGET_LAST:
				if($gallery_id=$wpdb->get_var("SELECT id FROM $table_name ORDER BY id DESC LIMIT 1"))
						$answer=muds_widget_photo($gallery_id,LAST_PHOTO, $instance['size']);
				break;
			case MUDS_WIDGET_RANDOMLAST:
				if($gallery_id=$wpdb->get_var("SELECT id FROM $table_name ORDER BY id DESC LIMIT 1"))
						$answer=muds_widget_photo($gallery_id,RANDOM_PHOTO, $instance['size']);
				break;
			case MUDS_WIDGET_RANDOMFROM:
				$gallery = $instance['gallery_id'];
				if(empty($gallery)) $gallery = $instance['gallery']; //A stupid bug
				if(muds_get_gallery_file($gallery))
					$answer=muds_widget_photo($gallery,RANDOM_PHOTO, $instance['size']);
				break;
		}
	} else { //We need minimax.
		$answer = sprintf(__('You have to install <a href="%s" target="_BLANK">minimax %1.1f</a> in order for this plugin to work.', 'mudslide'), "http://wordpress.org/extend/plugins/minimax/", MUDS_MNMX_V);
	}
	return $answer;
}

// check version. only 2.8 WP support class multi widget system
global $wp_version;
if((float)$wp_version >= 2.8) { //The new widget system

	class MudsWidget extends WP_Widget {
		
		/**
		 * constructor
		 */	 
		function MudsWidget() {
			parent::WP_Widget('mudslide', 'MudSlideShow', array('description' => __('MudSlideShow widget to show photos from the galleries you have added into your posts.', 'mudslide') ));	
		}
		
		/**
		 * display widget
		 */	 
		function widget($args, $instance) {
		
			$sidebars_widgets = get_option('sidebars_widgets');
		
			extract($args, EXTR_SKIP);
			echo $before_widget;
			$title = empty($instance['title']) ? '&nbsp;' : apply_filters('widget_title', $instance['title']);
			if ( !empty( $title ) ) { echo $before_title . $title . $after_title; };
			echo muds_widget_contents($instance);
			echo $after_widget;
			
		}
		
		/**
		 *	update/save function
		 */	 	
		function update($new_instance, $old_instance) {
			$instance = $old_instance;
			$instance['title'] = strip_tags($new_instance['title']);
			$instance['type'] = strip_tags($new_instance['type']);
			if($instance['type']==MUDS_WIDGET_RANDOMFROM) {
				$instance['gallery_id']=strip_tags($new_instance['gallery_id']);
			} else {
				$instance['gallery_id']=0;
			}
			$instance['size'] = strip_tags($new_instance['size']);
			
			return $instance;
		}
		
		/**
		 *	admin control form
		 */	 	
		function form($instance) {
			$default = 	array('title'=> __('Photo Album' , 'mudslide'), 'type'=>'1', 'gallery_id' => '0', 'size' => 160);
			$instance = wp_parse_args( (array) $instance, $default );
			
			$title = $instance['title'];
			$title_name = $this->get_field_name('title');
			$title_id = $this->get_field_id('title');
			
			$type = $instance['type'];
			$type_name = $this->get_field_name('type');
			$type_id = $this->get_field_id('type');
			
			$size = $instance['size'];
			$size_name = $this->get_field_name('size');
			$size_id_name = $this->get_field_id('size');
			
			$gallery = $instance['gallery_id'];
			if(empty($gallery)) $gallery = $instance['gallery']; //A stupid bug
			$gallery_name=$this->get_field_name('gallery_id');
			$gallery_id=$this->get_field_id('gallery_id');
			include('templates/mudslide_widget_manage.php');
		}
	}

	/* register widget when loading the WP core */
	add_action('widgets_init', muds_register_widgets);

	function muds_register_widgets(){
		// curl need to be installed
		register_widget('MudsWidget');
	}

}

?>
